﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
//using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using ScriptPortal.Vegas;

namespace JETDV_Render_Script
{
    public partial class Form1 : Form
    {
        public Vegas myVegas;
        private Renderer[] FullRenderer = new Renderer[999];
        private RenderTemplate[] FullTemplate = new RenderTemplate[999];

        private bool GotFileNames = false;
        private string[] fileNames;

        public Form1(Vegas vegas)
        {
            myVegas = vegas;

            InitializeComponent();

            FindAllRenderers();
        }

        public void FindAllRenderers()
        {
            int RCount = 0;

            try
            {
                foreach (Renderer renderer in myVegas.Renderers)
                {
                    try
                    {
                        foreach (RenderTemplate renderTemplate in renderer.Templates)
                        {
                            if (renderTemplate.IsValid())
                            {
                                FullRenderer[RCount] = renderer;
                                FullTemplate[RCount] = renderTemplate;
                                cmbRenderType.Items.Add(renderer.Name + " - " + renderTemplate.Name);
                                RCount++;
                            }
                        }
                    }
                    catch
                    {
                    }
                }
            }
            catch
            {
            }

            if (cmbRenderType.SelectedIndex < 0)
            {
                cmbRenderType.SelectedIndex = 0;
            }
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            txtDirBox.Text = GetSaveDir(txtDirBox.Text);
        }

        private void btnRender_Click(object sender, EventArgs e)
        {
            Renderer myRenderer = FullRenderer[cmbRenderType.SelectedIndex];
            RenderTemplate myTemplate = FullTemplate[cmbRenderType.SelectedIndex];

            if (rdoFull.Checked)
            {
                string FullFileName = txtDirBox.Text + txtFileName.Text + myRenderer.FileExtension.Substring(1, 4);

                Timecode RStart = new Timecode("00:00:00:00");
                Timecode RLength = myVegas.Project.Length;

                DoRender(FullFileName, myRenderer, myTemplate, RStart, RLength, false, false);
            }

            if (rdoRegion.Checked)
            {
                foreach(Region myRegion in myVegas.Project.Regions)
                {
                    string FullFileName = txtDirBox.Text + "Region " + myRegion.Index + " " + txtFileName.Text + myRenderer.FileExtension.Substring(1, 4);

                    Timecode RStart = myRegion.Position;
                    Timecode RLength = myRegion.Length;

                    DoRender(FullFileName, myRenderer, myTemplate, RStart, RLength, false, false);
                }
            }

            if (rdoSpecial.Checked)
            {
                DoSpecialRender(myRenderer, myTemplate);
            }
            if (rdoProjects.Checked)
            {
                renderProjects(myRenderer, myTemplate);
            }
            if (rdoSnapshot.Checked)
            {
                exportImage();
            }
        }

        private void exportImage()
        {
            // save original project and preview settings
            VideoRenderQuality origPreviewRenderQuality = myVegas.Project.Preview.RenderQuality;
            bool origPreviewFillSize = myVegas.Project.Preview.FullSize;
            VideoFieldOrder origFieldOrder = myVegas.Project.Video.FieldOrder;
            VideoDeinterlaceMethod origProjectDeinterlaceMethod = myVegas.Project.Video.DeinterlaceMethod;

            // Set the preview quality and size. 
            myVegas.Project.Preview.RenderQuality = VideoRenderQuality.Best;
            myVegas.Project.Preview.FullSize = true;
            // Set the field order and deinterlace method 
            myVegas.Project.Video.FieldOrder = VideoFieldOrder.ProgressiveScan;
            myVegas.Project.Video.DeinterlaceMethod = VideoDeinterlaceMethod.InterpolateFields;

            myVegas.UpdateUI();

            string FullFileName = txtDirBox.Text + txtFileName.Text + ".png";
            myVegas.SaveSnapshot(FullFileName, ImageFileFormat.PNG, myVegas.Transport.CursorPosition);

            // restore the project and preview settings 
            myVegas.Project.Preview.RenderQuality = origPreviewRenderQuality;
            myVegas.Project.Preview.FullSize = origPreviewFillSize;
            myVegas.Project.Video.FieldOrder = origFieldOrder;
            myVegas.Project.Video.DeinterlaceMethod = origProjectDeinterlaceMethod;
        }


        private void renderProjects(Renderer myRenderer, RenderTemplate myTemplate)
        {
            if (!GotFileNames)
            {
                MessageBox.Show("You must get the files to render before you can choose this option. Please press the 'Select Projects' button.");
                return;
            }

            // Read the files
            foreach (String file in fileNames)
            {
                myVegas.NewProject(false, false);
                myVegas.OpenProject(file);
                string FullFileName = txtDirBox.Text + Path.GetFileNameWithoutExtension(file) + "-" + txtFileName.Text + myRenderer.FileExtension.Substring(1, 4);

                Timecode RStart = new Timecode("00:00:00:00");
                Timecode RLength = myVegas.Project.Length;

                DoRender(FullFileName, myRenderer, myTemplate, RStart, RLength, false, false);
            }
            myVegas.NewProject(false, false);
        }

        public void DoSpecialRender(Renderer myRenderer, RenderTemplate myTemplate)
        {
            // Mute all tracks
            foreach (Track myTrack in myVegas.Project.Tracks)
            {
                myTrack.Mute = true;
            }

            // Do all of the renders
            foreach (Track myTrack in myVegas.Project.Tracks)
            {
                myTrack.Mute = false;

                int TrackNum = myTrack.Index + 1;

                foreach (Region myRegion in myVegas.Project.Regions)
                {
                    int RegionNum = myRegion.Index + 1;
                    string FullFileName = txtDirBox.Text;
                    if (myRegion.Label == "")
                    {
                        FullFileName = FullFileName + "Region " + RegionNum;
                    }
                    else
                    {
                        FullFileName = FullFileName + myRegion.Label;
                    }
                    FullFileName = FullFileName + " Track" + TrackNum + " " + txtFileName.Text + myRenderer.FileExtension.Substring(1, 4);

                    Timecode RStart = myRegion.Position;
                    Timecode RLength = myRegion.Length;

                    DoRender(FullFileName, myRenderer, myTemplate, RStart, RLength, false, false);

                }

                myTrack.Mute = true;
            }

            // UnMute all tracks
            foreach (Track myTrack in myVegas.Project.Tracks)
            {
                myTrack.Mute = false;
            }

        }

        public string GetSaveDir(string OrgSaveDir)
        {
            if (OrgSaveDir == null)
            {
                OrgSaveDir = "";
            }
            FolderBrowserDialog saveFileDialog = new FolderBrowserDialog();
            saveFileDialog.Description = "Select the desired folder";
            saveFileDialog.ShowNewFolderButton = true;
            if (!(OrgSaveDir == ""))
            {
                string initialDir = OrgSaveDir;
                if (Directory.Exists(initialDir))
                {
                    saveFileDialog.SelectedPath = initialDir;
                }
            }
            else
            {
                saveFileDialog.SelectedPath = "c:\\";
            }
            if (System.Windows.Forms.DialogResult.OK == saveFileDialog.ShowDialog())
            {
                return Path.GetFullPath(saveFileDialog.SelectedPath) + Path.DirectorySeparatorChar;
            }
            else
            {
                return OrgSaveDir;
            }

        }

        public void DoRender(string fileName, Renderer rndr, RenderTemplate rndrTemplate, Timecode start, Timecode length, bool IncMarkers, bool SetStretch)
        {
            // make sure the file does not already exist
            bool overwriteExistingFiles = true;

            if (!overwriteExistingFiles && File.Exists(fileName))
            {
                MessageBox.Show("File already exists: " + fileName);
                return;
            }

            //MessageBox.Show(fileName);

            if (File.Exists(fileName))
            {
                try
                {
                    File.Delete(fileName);
                }
                catch
                {
                }
            }

            if (File.Exists(fileName))
            {
                try
                {
                    myVegas.Project.MediaPool.Remove(fileName);
                    myVegas.UpdateUI();
                    File.Delete(fileName);
                }
                catch
                {
                }
            }

            if (File.Exists(fileName))
            {
                MessageBox.Show("Could not delete: " + fileName);
                return;
            }

            // perform the render. The Render method returns
            // a member of the RenderStatus enumeration. If
            // it is anything other than OK, exit the loops.
            RenderArgs args = new RenderArgs();
            args.OutputFile = fileName;
            args.RenderTemplate = rndrTemplate;
            args.Start = start;
            args.Length = length;
            args.IncludeMarkers = IncMarkers;
            args.StretchToFill = SetStretch;

            RenderStatus status = myVegas.Render(args);

            //RenderStatus status = Vegas.Render(fileName, rndrTemplate, start, length);

            // if the render completed successfully, just return
            if (status == RenderStatus.Complete)
                return;

            // if the user canceled, throw out a special message that won't be
            // displayed.
            if (status == RenderStatus.Canceled)
            {
                MessageBox.Show("User canceled");
                return;
            }

            // if the render failed, throw out a detailed error message.
            StringBuilder msg = new StringBuilder("Render failed:\n");
            msg.Append("\n file name: ");
            msg.Append(fileName);
            msg.Append("\n Renderer: ");
            msg.Append(rndr.FileTypeName);
            msg.Append("\n Template: ");
            msg.Append(rndrTemplate.Name);
            msg.Append("\n Start Time: ");
            msg.Append(start.ToString());
            msg.Append("\n Length: ");
            msg.Append(length.ToString());
            MessageBox.Show(Convert.ToString(msg));
        }

        private void btnProjects_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "VEGAS Projects (*.VEG)|*.VEG|All files (*.*)|*.*";
            ofd.Multiselect = true;
            ofd.Title = "Select the VEGAS Projects to render";
            DialogResult dr = ofd.ShowDialog();
            if (dr == System.Windows.Forms.DialogResult.OK)
            {
                fileNames = ofd.FileNames;
                GotFileNames = true;
            }
            else
            {
                GotFileNames = false;
            }
        }
    }
}

 public class EntryPoint
{
    private static JETDV_Render_Script.Form1 form;

    public void FromVegas(Vegas vegas)
    {
        form = new JETDV_Render_Script.Form1(vegas);
        form.ShowDialog();
    }
}

